<?php

class VodCategoriesScreen extends AbstractPreloadedRegularScreen
    implements UserInputHandler
{
    const ID = 'vod_categories';

    public static function get_media_url_str()
    {
        return MediaURL::encode(array('screen_id' => self::ID));
    }

    ///////////////////////////////////////////////////////////////////////

    private $vod;
    public $session;

    public function __construct(Vod $vod, $session)
    {
        $this->vod = $vod;
        $this->session = $session;

        parent::__construct(self::ID,
            $vod->get_vod_categories_folder_views());

        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        $select_category_action =
            UserInputHandlerRegistry::create_action(
                $this, 'select_category');

        return array
        (
            GUI_EVENT_KEY_ENTER => $select_category_action,
        );
    }

    public function get_handler_id()
    {
        return self::ID;
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        hd_print('Vod categories: handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");

        if ($user_input->control_id == 'select_category') {
            if (!isset($user_input->selected_media_url))
                return null;

            $media_url = MediaURL::decode($user_input->selected_media_url);
            $category_id = $media_url->category_id;
            $caption = $this->vod->get_category_caption($category_id);
            $media_url_str = $this->vod->get_category_media_url_str($category_id);

            return ActionFactory::open_folder($media_url_str, $caption);
        }

        return null;
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies)
    {
        $this->vod->folder_entered($media_url, $plugin_cookies);

        $this->vod->ensure_categories_loaded($plugin_cookies);

        $category_ids = $this->vod->get_category_ids();

        $items = array();

        foreach ($category_ids as $category_id) {

            if ($this->session->api_vod_list_categories($category_id, null, null)->total < 1) continue;

            $caption = $this->vod->get_category_caption($category_id);
            $media_url_str = $this->vod->get_category_media_url_str($category_id);
            $icon_url = $this->vod->get_category_icon_url($category_id);

            $items[] = array
            (
                PluginRegularFolderItem::media_url => $media_url_str,
                PluginRegularFolderItem::caption => $caption,
                PluginRegularFolderItem::view_item_params => array
                (
                    ViewItemParams::icon_path => $icon_url,
                )
            );
        }

        return $items;
    }

    public function get_archive(MediaURL $media_url)
    {
        return $this->vod->get_archive($media_url);
    }
}

?>
