<?php

class VodGenresScreen extends AbstractPreloadedRegularScreen
    implements UserInputHandler
{
    const ID = 'vod_genres';

    public static function get_media_url_str($page_name = null, $arg = null)
    {
        $arr['screen_id'] = self::ID;
        $arr['page_name'] = $page_name;
        if ($page_name === 'year')
            $arr['year'] = $arg;
        return MediaURL::encode($arr);
    }

    ///////////////////////////////////////////////////////////////////////

    private $vod;
    public $session;

    public function __construct(Vod $vod, $session)
    {
        $this->vod = $vod;
        $this->session = $session;

        parent::__construct(self::ID,
            $vod->get_vod_genres_folder_views());

        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_action_map(MediaURL $media_url, &$plugin_cookies)
    {
        $select_genre_action =
            UserInputHandlerRegistry::create_action(
                $this, 'select_genre');

        return array
        (
            GUI_EVENT_KEY_ENTER => $select_genre_action,
        );
    }

    public function get_handler_id()
    {
        return self::ID;
    }

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        hd_print('Vod genres: handle_user_input:');
        foreach ($user_input as $key => $value)
            hd_print("  $key => $value");

        if ($user_input->control_id == 'select_genre') {
            if (!isset($user_input->selected_media_url))
                return null;

            $media_url = MediaURL::decode($user_input->selected_media_url);
            $parent_media_url = MediaURL::decode($user_input->parent_media_url);
            $genre_id = $media_url->genre_id;
            $caption = $this->vod->get_genre_caption($genre_id);

            if ($parent_media_url->page_name == 'year')
            {
                $media_url_str = $this->vod->get_genre_and_year_media_url_str($genre_id, $parent_media_url->year);
            }
            else
            {
                $media_url_str = $this->vod->get_genre_media_url_str($genre_id);
            }

            return ActionFactory::open_folder($media_url_str, $caption);
        }

        return null;
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_all_folder_items(MediaURL $media_url, &$plugin_cookies)
    {
        $this->vod->folder_entered($media_url, $plugin_cookies);

        $this->vod->ensure_genres_loaded($plugin_cookies);

        $genre_ids = $this->vod->get_genre_ids();

        $items = array();

        if (isset($media_url->year))
        {
            $items[] = array
            (
                PluginRegularFolderItem::media_url => 'All',
                PluginRegularFolderItem::caption => 'All',
                PluginRegularFolderItem::view_item_params => array
                (
                    ViewItemParams::icon_path => $this->vod->get_genre_icon_url('all'),
                )
            );
        }

        foreach ($genre_ids as $genre_id) {

            if (isset($media_url->year))
            {
                if ($this->session->api_vod_list_by_genre_and_year($genre_id, $media_url->year, null, null)->total < 1)
                    continue;
            }
            else
            {
                if ($this->session->api_vod_list_genres($genre_id, null, null)->total < 1) continue;
            }

            $caption = $this->vod->get_genre_caption($genre_id);
            $media_url_str = $this->vod->get_genre_media_url_str($genre_id);
            $icon_url = $this->vod->get_genre_icon_url($genre_id);

            $items[] = array
            (
                PluginRegularFolderItem::media_url => $media_url_str,
                PluginRegularFolderItem::caption => $caption,
                PluginRegularFolderItem::view_item_params => array
                (
                    ViewItemParams::icon_path => $icon_url,
                )
            );
        }

        return $items;
    }

    public function get_archive(MediaURL $media_url)
    {
        return $this->vod->get_archive($media_url);
    }
}

?>

